<?php
/**
 * Clase abstracta para los adaptadores. Contiene las definiciones
 * base para la creación de nuevos adaptadores.
 * 
 * @package PAR
 * @author Manuel A. Rubio
 * @license GPL
 */
abstract class PAR_Adapters {

	/**
	 * Función para determinar al conversión de tipo
	 * del estándar SQL al tipo que emplea la base de
	 * datos específica.
	 *
	 * @param string $type tipo de dato SQL estándar.
	 * @param integer $size tamaño para tipos de datos como el de texto.
	 * @return string el tipo de dato o null en caso de no encontrarse.
	 */
	public function type( $type, $size = 255 ) { 
		return null;
	}

	/**
	 * Función para poder tomar una columna de tipo fecha
	 * de forma adecuada (según si tiene formato UTC o cualquier otro).
	 *
	 * @param string $field el nombre del campo.
	 * @return string el nombre del campo.
	 */
	public function select_date( $field ) {
		return $this->get_table_name($field);
	}
	
	/**
	 * Función para retornar el método de cada
	 * sistema gestor de base de datos para obtener
	 * la fecha actual.
	 *
	 * @return string la forma de recuperar la fecha actual.
	 */
	public function date_current() {
		return "CURRENT_DATE";
	}
	
	/**
	 * Función para retornar el método de cada
	 * sistema gestor de base de datos para obtener
	 * la hora actual.
	 *
	 * @return string la forma de recuperar la hora actual.
	 */
	public function time_current() {
		return "CURRENT_TIME";
	}

	/**
	 * Función para retornar el método de cada
	 * sistema gestor de base de datos para obtener
	 * la fecha y hora actual.
	 *
	 * @return string la forma de recuperar la fecha y hora actual.
	 */
	public function datetime_current() {
		return "CURRENT_DATETIME";
	}

	/**
	 * Función que retorna el nombre de la clase, junto
	 * con el esquema en el que está definido.
	 *
	 * @param string $schema el nombre del esquema.
	 * @param string $name el nombre de la tabla.
	 * @return string el nombre de la tabla con esquema incluído.
	 */
	public function get_table_name( $schema, $name ) {
		return $schema . "." . $name;
	}

	/**
	 * Función para retornar el nombre de la columna.
	 *
	 * @param string $name nombre de la columna.
	 * @return string el nombre de la columna parseada.
	 */
	public function get_column_name( $name ) {
		return $name;
	}
}

/**
 * Función que carga todos los adaptadores que haya
 * en la carpeta específica de adaptadores.
 */
if ($d = opendir(__DIR__ . "/Adapters")) {
	while ($file = readdir($d)) {
		if (eregi(".php$", $file)) {
			include_once(__DIR__ . "/Adapters/$file");
		}
	}
	closedir($d);
}
