<?php
/**
 * Esta clase es la base del sistema, es de la que heredarán todas
 * las clases que se mapeen a tablas de las base de datos, tiene
 * las funciones básicas.
 *
 * @package PAR
 * @author Manuel A. Rubio
 * @license GPL
 */
class PAR_Base extends PAR_Connection  {

    /**
     * @var mixed los atributos de la tabla para ser almacenados o rescatados.
     */
    protected $attrs;

    /**
     * @var boolean indica si la tupla que se está usando es nueva o no, para
     * que save sepa lo que hacer.
     */
    private $is_new;

    /**
     * Constructor. Define los valores base que serán usados por los objetos
     * hijos.
     */
    public function __construct() {
		parent::__construct();
        $t = static::$table;
        self::try_connect();
        $this->attrs = array();
        $this->is_new = true;
    }

    /**
     * Función especial que rescata un atributo de la base de datos usando la
     * sintaxis acortada. P.ej.: $obj->nombre;
     *
     * @param string $name nombre del atributo a rescatar.
     * @return string el valor contenido en un atributo de la tabla en curso.
     */
    public function __get( $name ) {
        return $this->attrs[$name];
    }

    /**
     * Función especial que establece un valor, pasado como parámetro, a un
     * atributo, también pasado como parámetro, usando la sintaxis acortada.
     * P. ej.: $obj->nombre = "Manuel";
     *
     * @param string $name nombre del atributo a rescatar.
     * @param string $value el valor del atributo a establecer.
     */
    public function __set( $name, $value ) {
        $this->attrs[$name] = $value;
    }

    /**
     * Rescata un objeto de la base de datos, dado su identificador.
     *
     * @param string $id identificador del objeto a rescatar.
     * @return PAR_Base retorna un PAR_Base con los datos solicitados o null.
     */
    public static function find_by_id( $id ) {
        self::try_connect();
        $con = self::getInstancePDO(static::$idx);
        $id = static::quote("id", $id);
        $sql = "SELECT * FROM " . static::$table . " WHERE " . static::$id . " = $id";
        $sth = $con->query($sql);
        $row = ($sth instanceof PDOStatement) ? $sth->fetch(PDO::FETCH_ASSOC) : array();
        return self::populate($row);
    }

    /**
     * Retorna el número de registros que están en la base de datos, dadas
     * las condiciones, o globalmente si no existen.
     *
     * @param mixed $cond un array con las condiciones o una cadena de texto tipo SQL.
     * @return integer el número de registros encontrados.
     */
    public static function count( $cond = null ) {
		return self::stats("*", "COUNT", $cond);
	}
	
	public static function sum( $field, $cond = null ) {
		return self::stats($field, "SUM", $cond);
	}
	
	public static function max( $field, $cond = null ) {
		return self::stats($field, "MAX", $cond);
	}
	
	public static function min( $field, $cond = null ) {
		return self::stats($field, "MIN", $cond);
	}
	
	public static function avg( $field, $cond = null ) {
		return self::stats($field, "AVG", $cond);
	}
	
	protected static function stats( $field = "*", $operation = "COUNT", $cond = null ) {
        self::try_connect();
        $con = self::getInstancePDO(static::$idx);
        $sql = "SELECT $operation($field) FROM " . static::$table;
        $params = array();
        if ($cond != null) {
	 		if (is_array($cond)) {
	            foreach ($cond as $name => $value) {
	                $op = "=";
	                if (is_array($value)) {
	                    $op = $value["op"];
	                    $value = $value["value"];
	                }
	                $params[] = $name . $op . static::quote($name, $value);
	            }
	            if (count($params) > 0) {
	                $sql .= " WHERE " . join(" AND ", $params);
	            }
	        } else {
            	$sql .= " WHERE $cond";
        	}
		}
        $sth = $con->query($sql);
        return ($sth instanceof PDOStatement) ? $sth->fetchColumn() : 0;
    }

    /**
     * Rescata tuplas de la base de datos en forma de objetos de tipo PAR_Base,
     * según las condiciones (cond) pasadas (p.ej.: "nombre = 'Manuel'"), el
     * orden que se indique (p.ej.: "apellidos asc, nombre asc") y el límite de
     * tuplas a rescatar.
     *
     * @param string $cond (opcional) las condiciones a aplicar en la búsqueda.
     * @param string $order (opcional) el orden que aplicar al listado.
     * @param string $limit (opcional) el número de tuplas a rescatar.
     * @return mixed un array de PAR_Base o vacío (ver populate).
     */
    public static function find( $cond = null, $order = null, $limit = null ) {
        self::try_connect();
        $limit_mssql = "";
        $limit_other = "";
        if ($limit != null) {
            $conf = self::$conf_a[static::$idx];
            if ($conf["driver"] == "mssql") {
                $limit_mssql = "TOP $limit ";
            } else {
                $limit_other = " LIMIT $limit";
            }
        }
        $con = self::getInstancePDO(static::$idx);
        $sql = "SELECT $limit_mssql* FROM " . static::$table;
        $params = array();
        if ($cond != null and is_array($cond)) {
            foreach ($cond as $name => $value) {
                $op = "=";
                if (is_array($value)) {
                    $op = $value["op"];
                    $value = $value["value"];
                }
                $params[] = $name . "=" . static::quote($name, $value);
            }
            if (count($params) > 0) {
                $sql .= " WHERE " . join(" AND ", $params);
            }
        } elseif ($cond != null) {
            $sql .= " WHERE $cond";
        }
        if ($order != null) {
            $sql .= " ORDER BY " . $order;
        }
        $sql .= $limit_other;
        $sth = $con->query($sql);
        $row = ($sth instanceof PDOStatement) ? $sth->fetchAll(PDO::FETCH_ASSOC) : array();
        return self::populate($row, true);
    }

    /**
     * Usando los datos pasados como parámetro, se encarga de crear tantos
     * objetos de tipo PAR_Base (sus herederos, más bien) como haga falta
     * y va asignando los datos pertinentes a cada uno.
     *
     * @param array $row los datos de la tupla o tuplas.
     * @param boolean $multi indica si row es un vector (false) o una matrix (true).
     * @return mixed los datos almacenados en su correspondiente forma.
     */
    private static function populate( $row, $multi = false ) {
        if (is_array($row)) {
            $class = static::$class;
            if ($multi) {
                $o = array();
                foreach ($row as $name => $value) {
                    $p = new $class();
                    $p->is_new = false;
                    foreach ($value as $n => $v) {
                        $p->$n = $v;
                    }
                    $p->after_populate();
                    $o[] = $p;
                }
            } else {
                $o = new $class();
                $o->is_new = false;
                foreach ($row as $name => $value) {
                    $o->$name = $value;
                }
                $o->after_populate();
            }
            return $o;
        }
        return null;
    }

    /**
     * Función a sobrecargar en caso de que se quiera realizar algún tratamiento
     * sobre los datos recién rescatados (de la base de datos).
     */
    protected function after_populate() {
        return true;
    }

	/**
	 * Función interna. Se usa para descomponer el nombre de las funciones
	 * inteligentes (p.ej.: find_by_*, count_by_*, ...) a modo de poder crear
	 * una llamada a función "pseudocodificada".
	 *
	 * @param PDO &$con la conexión a la base de datos que se usará (para quote).
	 * @param mixed $parts las partes de la cadena separadas por "_".
	 * @param mixed $params los parámetros pasados para la creación del where.
	 * @return string cadena del where para usar directamente en el SQL.
	 */
	private static function make_where( &$con, $parts, $params ) {
		$names = array();
		$name = "";
		$j = 0;
		foreach ($parts as $i => $part) {
			switch ($part) {
				case "and": 
				case "or":
				case "not":
					if (strlen($name) > 0) {
						$names[] = "$name = " . static::quote($name, $params[$j++]);
					}
					$names[] = strtoupper(trim($part));
					$name = "";
					break;
				default:
					$name = (strlen($name) > 0) ? $name . "_" . $part : $part;
			}
		}
		if (strlen($name) > 0) {
			$names[] = "$name = " . static::quote($name, $params[$j++]);
		}
		return join(" ", $names);
	}

    /**
     * Función especial. Ejecuta las funciones del tipo:
     *
     *  - find_by_*: rescata una tupla, según el atributo a filtrar que se indique.
     *  - find_all_by_*: rescata todas las tuplas que casen con el atributo por el que filtrar.
     *  - count_by_*: retorna el total de tuplas según el criterio empleado.
     *
     * @param string $name nombre del método (find_by_* o find_all_by_*).
     * @param mixed $params los parámetros para cada método.
     * @return mixed según el caso.
     */
    public static function __callStatic( $name, $params ) {
        if (preg_match("/^find_by_/", $name)) {
            self::try_connect();
            $parts = explode("_", $name);
			array_shift($parts);
			array_shift($parts);
            $con = self::getInstancePDO(static::$idx);
			$where = self::make_where($con, $parts, $params);
            $sql = "SELECT * FROM " . static::$table . " WHERE $where";
            $sth = $con->query($sql);
            $row = ($sth instanceof PDOStatement) ? $sth->fetch(PDO::FETCH_ASSOC) : array();
            return self::populate($row);
        } elseif (preg_match("/^find_all_by_/", $name)) {
            self::try_connect();
            $parts = explode("_", $name);
			array_shift($parts);
			array_shift($parts);
			array_shift($parts);
            $con = self::getInstancePDO(static::$idx);
			$where = self::make_where($con, $parts, $params);
            $sql = "SELECT * FROM " . static::$table . " WHERE $where";
            $sth = $con->query($sql);
            $row = ($sth instanceof PDOStatement) ? $sth->fetchAll(PDO::FETCH_ASSOC) : array();
            return self::populate($row, true);
        } elseif (preg_match("/^count_by_/", $name)) {
            self::try_connect();
            $parts = explode("_", $name);
			array_shift($parts);
			array_shift($parts);
            $con = self::getInstancePDO(static::$idx);
			$where = self::make_where($con, $parts, $params);
            $sql = "SELECT COUNT(*) FROM " . static::$table . " WHERE $where";
            $sth = $con->query($sql);
            $row = ($sth instanceof PDOStatement) ? $sth->fetchColumn() : 0;
            return $row;
        } elseif (preg_match("/^delete_by_/", $name)) {
			self::try_connect();
			$parts = explode("_", $name);
			array_shift($parts);
			array_shift($parts);
            $con = self::getInstancePDO(static::$idx);
			$where = self::make_where($con, $parts, $params);
			$sql = "DELETE FROM " . static::$table . " WHERE $where";
			return $con->prepare($sql)->execute();
		}
        return null;
    }

	public static function quote($name, $value) {
		$con = self::getInstancePDO(static::$idx);
		if (isset(static::$quote[$name]) and static::$quote[$name] === false) {
			return $value;
		}
		return $con->quote($value);
	}

    /**
     * Guarda (inserta o actualiza) los datos del objeto.
     *
     * @return boolean verdadero si se ha realizado bien la escritura, falso sino.
     */
    public function save() {
        $attrs = array();
        foreach ($this->attrs as $name => $value) {
            if ($this->is_new) {
                $attrs["fields"][] = $name;
                $attrs["values"][] = static::quote($name, $value);
            } elseif ($name != static::$id) {
                $attrs["set"][] = "$name = " . static::quote($name, $value);
            } else {
                $attrs["id"] = "$name = " . static::quote($name, $value);
            }
        }
		if (!$this->before_save()) {
			return false;
		}
        if ($this->is_new) {
			if (!$this->before_create()) {
				return false;
			}
            $fields = join(",", $attrs["fields"]);
            $values = join(",", $attrs["values"]);
            $sql = "INSERT INTO " . static::$table . "($fields) VALUES ($values)";
        } else {
			if (!$this->before_update()) {
				return false;
			}
            $set = join(",", $attrs["set"]);
            $id = $attrs["id"];
            $sql = "UPDATE " . static::$table . " SET $set WHERE $id";
        }
        return $this->execute($sql);
    }

	/**
	 * Función ejecutada antes de realizar la operación de almacenamiento,
	 * ya sea una actualización o una inserción.
	 *
	 * @return boolean verdadero para continuar con "save" o falso para abortar.
	 */
	public function before_save() {
		return true;
	}
	
	/**
	 * Función ejecutada antes de realizar la operación de inserción (save).
	 *
	 * @return boolean verdadero para continuar con "save" o falso para abortar.
	 */
	public function before_create() {
		return true;
	}
	
	/**
	 * Función ejecutada antes de realizar la operación de actualización (save).
	 *
	 * @return boolean verdadero para continuar con "save" o falso para abortar.
	 */
	public function before_update() {
		return true;
	}

}
