<?php
/**
 * Clase encargada de establecer la conexión y
 * mantener el <em>pool</em> de conexiones. Así como
 * las configuraciones de las diferentes conexiones
 * con las base de datos que se pueden configurar.
 *
 * @package PAR
 * @author Manuel A. Rubio
 * @license GPL
 */
class PAR_Connection {
	
    /**
     * @var Log objeto de tipo PEAR::Log para soltar información.
     */
    protected static $log;

    /**
     * @var mixed almacena las conexiones con las base de datos.
     */
    protected static $conn_a = array();

    /**
     * @var mixed almacena las configuraciones para las conexiones con las
     * bases de datos.
     */
    protected static $conf_a = array();

	/**
	 * Constructor de la clase. Se encarga de configurar el log.
	 *
	 * @global string $log_file el nombre de fichero de log.
	 * @global string $log_level el nivel de log para el fichero.
	 */
	public function __construct() {
        global $log_file, $log_level;
        if (!static::$log and isset($log_file) and isset($log_level)) {
            static::$log =& Log::factory('file', $log_file, get_class($this));
            static::$log->setMask($log_level);
        }
	}
	
	public static function getInstancePDO($idx) {
		if (!self::$conf_a[$idx]) {
			throw new Exception("Configuration for $idx not found.");
		}
		self::try_connect($idx);
		if (!self::$conn_a[$idx]) {
			// FIXME: crear una excepción propia para esto.
			throw new Exception("Can't connect to $idx.");
		}
		return self::$conn_a[$idx];
	}
	
    /**
     * Realiza la conexión con la base de datos, usando PDO.
     *
     * @param string $idx nombre de la conexión
	 * @return boolean verdadero si la conexión se estableció o falso sino.
     */
    protected static function connect($idx) {
        $dbcon =& self::$conf_a[$idx];
        if (is_array($dbcon)) {
			$adapter = $dbcon["adapter"];
			$host = (empty($dbcon["host"])) ? null : $dbcon["host"];
			$database = $dbcon["database"];
			$username = (empty($dbcon["username"])) ? null : $dbcon["username"];
			$password = (empty($dbcon["password"])) ? null : $dbcon["password"];
            if ($adapter != "sqlite") {
                $con = new PDO("$adapter:host=$host;dbname=$database", $username, $password);
            } else {
                $con = new PDO("sqlite:$database");
            }
			if (!$con) {
				static::$log->log("La conexión no se estableció: $idx", PEAR_LOG_ERR);
				return false;
			}
            self::$conn_a[$idx] =& $con;
        } else {
            static::$log->log("No se pudo realizar la conexión: $idx", PEAR_LOG_ERR);
			return false;
        }
		return true;
    }

	/**
	 * Cuando se desea establecer una conexión ya existente (como por ejemplo en tests),
	 * es útil disponer de una función como esta, que establece la conexión a usar
	 * de una válida que se pase (que sea de tipo PDO).
	 *
	 * @param string $idx nombre de la conexión
	 * @param PDO $pdo conexión que se establece con el índice dado.
	 */
    public static function setConnection( $idx, PDO &$pdo ) {
        self::$conn_a[$idx] =& $pdo;
    }

    /**
     * Asigna la configuración de forma estática a PAR. Esta configuración
     * pasada como parámetro, se usará en todos los objetos de PAR.
     *
     * @param mixed $conf configuración de base de datos, general, para PAR.
     */
    public static function init( $conf ) {
        self::$conf_a = $conf;
    }

    /**
     * Intenta conectar con la base de datos. Se usa en caso de haber perdido
     * la conexión con la base de datos que se esté usando, o en caso de que
     * la conexión se haya cerrado por algún motivo, o no estuviese abierta.
     * 
     * @global string $log_file nombre del fichero de log.
     * @global string $log_level nivel de log para el fichero de log.
     * @param string idx (opcional) indica la conexión a usar explícitamente.
     */
    protected static function try_connect( $idx = null ) {
        global $log_file, $log_level;
        if (!static::$log and isset($log_file) and isset($log_level)) {
            static::$log =& Log::factory('file', $log_file, __CLASS__);
            static::$log->setMask($log_level);
        }
		$idx = ($idx === null) ? static::$idx : $idx;
        if (empty(self::$conn_a[$idx])) {
            self::connect($idx);
        }
    }

    /**
     * Ejecuta una consulta SQL con la conexión activa. Recomendado solo uso
     * de DDL, ya que no retorna nada más que verdadero o falso.
     *
     * @return boolean verdadero si la consulta salió bien, falso sino.
     */
    public function execute( $sql ) {
        self::try_connect();
		static::$log->log("SQL: $sql\n", PEAR_LOG_DEBUG);
        $con = self::$conn_a[static::$idx];
        static::$log->log($sql, PEAR_LOG_DEBUG);
        $stmt = $con->prepare($sql);
		if (!$stmt) {
	        static::$log->log("Consulta no válida: " . $sql, PEAR_LOG_ERR);
			throw new Exception("Consulta no válida: $sql");
		}
		$stmt->execute();
		return ($stmt->errorCode() == "00000");
    }

	/**
	 * Recupera el último error sucedido en la base de datos.
	 *
	 * @return string el texto del error o null en caso de que no haya nada.
	 */
	public function get_last_error() {
        $con = self::$conn_a[static::$idx];
		if ($con->errorCode() !== null) {
			$error = $con->errorInfo();
			return $error[2];
		}
		return null;
	}
}
