<?php
/**
 * Clase abstracta para crear las migraciones. Es la base
 * de cada migración creada. Tiene las funciones base necesarias
 * para crear las migraciones.
 *
 * @package PAR
 * @author Manuel A. Rubio
 * @license GPL
 */
abstract class PAR_Migration extends PAR_Connection {

	/**
	 * @var string $idx identificador para la conexión a la base de datos.
	 * @var string[] $statements conjunto de comandos a ejecutar.
	 */
	private $idx;
	private $statements = array();

	/**
	 * @var string[][] $migrations la clase y el fichero de la migración a realizar.
	 */
	private static $migrations = array();

	/**
	 * Busca las migraciones a realizar.
	 *
	 * @param string $dir directorio base en el que buscar.
	 * @return mixed el número de la migración a la que migrar o null en caso de no ser posible.
	 */
	public static function find( $dir ) {
		if ($d = @opendir($dir)) {
			self::$migrations = array();
			while ($file = readdir($d)) {
				if (preg_match("/^([0-9]+)_([A-Za-z0-9_]+).php/i", $file, $args)) {
					$version = $args[1];
					settype($version, "int");
					$class = str_replace(" ", "", ucwords(str_replace("_", " ", $args[2])));
					self::$migrations[$version] = array ( $class, $file );
				}
			}
			closedir($d);
		}
		if (count(self::$migrations) > 0) {
			$keys = array_keys(self::$migrations);
			return $keys[count($keys) - 1];
		}
		return null;
	}
	
	/**
	 * Retorna las migraciones encontradas anteriormente por "find",
	 * en formato de array para su posterior ejecución.
	 *
	 * @param integer $source migración actual, de la que se parte.
	 * @param integer $target migración a la que se quiere llegar.
	 * @return array las migraciones (listado), en un hash de "up" o "down".
	 */
	public static function go_to( $source, $target ) {
		if ($target > $source) {
			// up!
			$migrations["type"] = "up";
			$migrations["up"] = array();
			for ($i=$source+1; $i<=$target; $i++) {
				if (isset(self::$migrations[$i])) {
					$migrations["up"][$i] = self::$migrations[$i];
				}
			}
		} else {
			// down!
			$migrations["type"] = "down";
			$migrations["down"] = array();
			for ($i=$source; $i>$target; $i--) {
				if (isset(self::$migrations[$i])) {
					$migrations["down"][$i] = self::$migrations[$i];
				}
			}
		}
		return $migrations;
	}

	/**
	 * Ejecuta las migraciones.
	 *
	 * @param string $type el tipo de migración. Puede ser "up" o "down".
	 * @param array $migrations las migraciones retornadas por la función "go_to".
	 */
	public static function exec( $type, $migrations ) {
		foreach ($migrations as $v => $name) {
			list($class, $file) = $name;
			include("db/migrate/$file");
			$migration = new $class();
			$migration->run($type);
			if ($type == "up") {
				$m = new PAR_Migration_Base();
				$m->version = $v;
				if (!$m->save()) {
					throw new Exception($m->get_last_error());
				}
			} else {
				PAR_Migration_Base::delete_by_version($v);
			}
		}
	}

	/**
	 * Llama a una función implementada en las clases hijas
	 * (up o down) para obtener el conjunto de mandatos a
	 * ejecutar y después los realiza en una sola transacción.
	 *
	 * @param string $type tipo de conjunto de acciones a realizar.
	 * @return boolean verdadero si todo ha ido bien, falso sino.
	 */
	public function run( $type ) {
		switch ($type) {
			case "up":
			case "down":
				$this->$type();
				if (is_array($this->statements) and count($this->statements) > 0) {
					foreach ($this->statements as $stmt) {
						if (!$stmt->run()) {
							return false;
						}
					}
				}
				break;
		}
		return true;
	}
	
	/**
	 * Define las acciones a llevar a cabo para
	 * tener la nueva versión de la base de datos.
	 *
	 * @return boolean verdadero si todo ha ido bien, falso sino.
	 */
	abstract public function up();
	
	/**
	 * Define las acciones a llevar a cabo para
	 * tener dar marcha atrás a la actualización
	 * llevada a cabo sobre la base de datos.
	 *
	 * @return boolean verdadero si todo ha ido bien, falso sino.
	 */
	abstract public function down();

	/**
	 * Permite el uso de una conexión ya abierta, en lugar de
	 * establecer una nueva.
	 * 
	 * @param string $idx nombre de la conexión (índice hash de la configuración).
	 */
	public function set_connection( $idx ) {
		$this->idx = $idx;
	}
	
	/**
	 * Crea una tabla. Instancia un objeto de tipo Table, con un nombre
	 * y opcionalmente un esquema.
	 * 
	 * @param string $name nombre de la tabla a crear.
	 * @param string $schema (opcional) nombre del esquema donde crear la tabla.
	 * @return PAR_Migration_Table la instancia de la tabla creada.
	 */
	public function &create_table( $name, $schema  = "" ) {
		$table = new PAR_Migration_Table($this->idx, $name, $schema, PAR_Migration_Table::ACTION_NEW);
		$this->statements[] = &$table;
		return $table;
	}

	/**
	 * Modifica una tabla. Instancia un objeto de tipo Table indicando que
	 * la tabla existe, a modo de poder agregarle atributos, eliminar de la
	 * definición o modificar los ya existentes.
	 *
	 * @param string $name nombre de la tabla.
	 * @param string $schema (opcional) esquema donde está la tabla creada.
	 * @return PAR_Migration_Table la instancia de la tabla creada.
	 */
	public function &alter_table( $name, $schema =  "" ) {
		$table = new PAR_Migration_Table($this->idx, $name, $schema, PAR_Migration_Table::ACTION_ALTER);
		$this->statements[] = &$table;
		return $table;
	}
	
	/**
	 * Elimina una tabla.
	 *
	 * @param string $name nombre de la tabla.
	 * @param string $schema (opcional) esquema donde está la tabla creada.
	 */
	public function drop_table( $name, $schema = "" ) {
		$table = new PAR_Migration_Table($this->idx, $name, $schema, PAR_Migration_Table::ACTION_DESTROY);
		$this->statements[] = &$table;
	}
	
}
