<?php
/**
 * Clase encargada de la creación y modificación de
 * las tablas.
 *
 * @package PAR_Migration
 * @author Manuel A. Rubio
 * @license GPL
 */
class PAR_Migration_Table extends PAR_Connection {

	/** 
	 * @var string $name nombre de la tabla.
	 * @var string $schema nombre del esquema donde se encuentra la tabla.
	 * @var string $idx base de datos (indentificador) al que debe conectarse.
	 * @var string $_idx almacena la identificación de la base de datos en el objeto.
	 * @var boolean $nueva indica si la tabla debe crearse.
	 * @var boolean $destroy indica si la tabla debe de eliminarse.
	 * @var mixed $columns columnas de la tabla (para la creación o la modificación).
	 * @var PAR_Adapter $adapter clase del adaptador que se usará para acceder a la base de datos.
	 */
	private $name;
	private $schema;
 	protected static $idx;
	private $_idx;
	private $nueva;
	private $destroy;
	
	private $columns;
	public $adapter;
	
	const ACTION_ALTER = 0;
	const ACTION_NEW = 1;
	const ACTION_DESTROY = 2;

	/**
	 * Constructor de la clase. Se indica la conexión con la base de datos donde se creará
	 * o se modificará o se destruirá la tabla, el nombre de la tabla y, opcionalmente, el
	 * esquema y los flags de si se debe de crear, destruir o modificar la tabla.
	 *
	 * @param string $idx la conexión con la base de datos.
	 * @param string $name nombre de la tabla.
	 * @param string $schema (opcional) nombre del esquema.
	 * @param integer $flags (opcional) indicador de la acción que se realizará (ALTER por defecto).
	 */
	public function __construct( $idx, $name, $schema = null, $flags = self::ACTION_ALTER ) {
		$this->name = $name;
		$this->schema = $schema;
		self::$idx = $idx;
		$this->_idx = $idx;
		$this->columns = array();
		
		self::try_connect();
		$adapter_class = "PAR_Adapters_" . ucwords(self::$conf_a[$idx]["adapter"]);
		$this->adapter = new $adapter_class();
		
		switch ($flags) {
			case self::ACTION_NEW:
				$this->nueva = true;
				$this->destroy = false;
				$this->add_column("id", "pkey");
				break;
			case self::ACTION_DESTROY:
				$this->nueva = false;
				$this->destroy = true;
				break;
			case self::ACTION_ALTER:
				$this->nueva = false;
				$this->destroy = false;
				break;
		}
	}

	/**
	 * Agrega una columna a la tabla actual. Se retorna la propia tabla
	 * para poder seguir anexando funciones de tipo add_column u otras
	 * similares.
	 *
	 * @param string $name nombre de la columna.
	 * @param string $type (opcional) el tipo de dato (por defecto: string).
	 * @param string $default (opcional) el dato por defecto.
	 * @param boolean $null (opcional) si es nulo o no.
	 * @param integer $limit (opcional) en caso de cadena, el tamaño máximo de la misma.
	 * @return la referencia al propio objeto.
	 */
	public function &add_column( $name, $type = "string", $default = null, $null = true, $limit = 0 ) {
		$this->columns[] = array (
			"name" => $name,
			"type" => $this->adapter->type($type, $limit),
			"original_type" => $type,
			"null" => $null,
			"default" => $default
		);
		return $this;
	}
	
	/**
	 * Agrega una referencia a otra tabla, clave foránea,
	 * creando el tipo de dato según la convención de:
	 * esquema_tabla_id.
	 * 
	 * @param string $table nombre de la tabla.
	 * @param string $schema (opcional) nombre del esquema.
	 * @return la referencia al propio objeto.
	 */
	public function &references( $table, $schema = "" ) {
		$name = ($schema != null and trim($schema) != "") ? $schema . "_" : "";
		$name .= $table . "_id";
		return $this->add_column($name, "integer");
	}

	/**
	 * Ejecuta la consulta almacenada.
	 *
	 * @return el resultado de la ejecución de SQL.
	 */
	public function run() {
		$ret = true;
		if ($this->destroy) {
			$txt = "DROP TABLE " . $this->adapter->get_table_name($this->schema, $this->name);
			self::$idx = $this->_idx;
			$ret = $this->execute($txt);
		} elseif ($this->nueva) {
			$txt = "CREATE TABLE " . $this->adapter->get_table_name($this->schema, $this->name);
			$txt .= " (\n";
			$cols = array();
			foreach ($this->columns as $col) {
				$txt2 = "\t" . $col["name"] . " " . $col["type"];
				$txt2 .= ($col["null"]) ? "" : " NOT NULL";
				if ($col["original_type"] == "pkey") {
					// no se hace nada
				} elseif ($col["default"] != null) {
					$txt2 .= " DEFAULT " . self::$conn_a[$this->_idx]->quote($col["default"]);
				} elseif ($col["null"] === null) {
					$txt2 .= " DEFAULT NULL";
				}
				$cols[] = $txt2;
			}
			$txt .= join(",\n", $cols) . "\n)";
			self::$idx = $this->_idx;
			$ret = $this->execute($txt);
		} else {
			foreach ($this->columns as $col) {
				$txt = "ALTER TABLE " . $this->adapter->get_table_name($this->schema, $this->name);
				$txt.= " ADD COLUMN ";
				$txt .= $col["name"] . " " . $col["type"];
				$txt .= ($col["null"]) ? " NULL" : " NOT NULL";
				if ($col["default"] != null) {
					$txt .= " DEFAULT " . self::$conn_a[$this->_idx]->quote($col["default"]);
				} elseif ($col["null"]) {
					$txt .= " DEFAULT NULL";
				}
				self::$idx = $this->_idx;
				$ret = ($ret and $this->execute($txt));
			}
		}
		if (self::$conf_a[static::$idx]["adapter"] == "sqlite" and !$this->destroy) {
			$this->execute("ANALYZE $this->name");
			print "Analizando {$this->name}\n";
		}
		return $ret;
	}
}
